# Copyright 2010-2011 Canonical Ltd.
from cStringIO import StringIO
from datetime import datetime

from mocker import MockerTestCase
from django.core.management.base import CommandError

from offspring.web.queuemanager.metrics import get_build_result_metrics
from offspring.web.queuemanager.management.commands.build_metrics import (
    Command)


class TestBuildResultMetricsCommand(MockerTestCase):

    def test_calls_get_build_result_metrics(self):
        """
        When called, the build_metrics command should fetch the list of metrics
        available for offspring.web.queuemanager and output them.
        """
        build_result_metrics_mock = self.mocker.replace(
            get_build_result_metrics)
        build_result_metrics_mock()
        self.mocker.result([("Description of value", 200)])
        self.mocker.replay()

        command = Command()
        stdout = StringIO()
        command.execute(stdout=stdout)

        self.assertEqual("Description of value = 200\n", stdout.getvalue())

    def test_calls_get_build_result_metrics_with_start_and_end(self):
        """
        If --start and --end are provided, we should use them to calculate the
        period of the metrics.
        """
        build_result_metrics_mock = self.mocker.replace(
            get_build_result_metrics)
        build_result_metrics_mock(
            datetime(2013, 06, 05, 0, 0, 0),
            datetime(2013, 06, 05, 23, 59, 59))

        self.mocker.result([("Description of value", 200)])
        self.mocker.replay()

        command = Command()
        stdout = StringIO()
        command.execute(start="2013-06-05", end="2013-06-05", stdout=stdout)
        self.assertEqual("Description of value = 200\n", stdout.getvalue())

    def test_error_with_missing_end_parameters(self):
        """
        If --start is provided without --end, then we should get an
        appropriate error.
        """
        exit_mock = self.mocker.replace("sys.exit")
        exit_mock(1)
        self.mocker.replay()

        command = Command()
        stderr = StringIO()
        command.execute(start="2013-06-05", stderr=stderr)

        self.assertTrue(
            "Error: --start and --end must be supplied" in stderr.getvalue())

    def test_error_with_missing_start_parameters(self):
        """
        If --end is provided without --start, then we should get an
        appropriate error.
        """
        exit_mock = self.mocker.replace("sys.exit")
        exit_mock(1)
        self.mocker.replay()

        command = Command()
        stderr = StringIO()
        command.execute(end="2013-06-05", stderr=stderr)

        self.assertTrue(
            "Error: --start and --end must be supplied" in stderr.getvalue())

    def test_error_with_end_before_start(self):
        """
        If the end date is not before the start datae we should get an error.
        """
        exit_mock = self.mocker.replace("sys.exit")
        exit_mock(1)
        self.mocker.replay()

        command = Command()
        stderr = StringIO()
        command.execute(start="2013-06-05", end="2013-05-05", stderr=stderr)

        self.assertTrue(
            "Error: start date must be before end date" in stderr.getvalue())
