# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import re
from datetime import datetime, timedelta, time

import dateutil.parser

from offspring.web.queuemanager.models import BuildResult


def get_build_result_metrics(start=None, end=None):
    """
    Return some simple metrics, in a format suitable for iterating over and
    displaying in a command-line tool.
    """
    results = []
    start_date = datetime.utcnow()

    seven_days = (start_date - timedelta(days=7), start_date)
    thirty_days = (start_date - timedelta(days=30), start_date)
    ninety_days = (start_date - timedelta(days=90), start_date)

    build_types = [("scheduled", True), ("requested", False)]

    if start and end:
        days = calculate_days(start, end)
        build_periods = [("%d days" % days, (start, end))]
    else:
        build_periods = [("7 days", seven_days), ("30 days", thirty_days),
                         ("90 days", ninety_days)]
    metrics = [(BuildResult.metrics.get_average_build_time, "build"),
               (BuildResult.metrics.get_average_queue_time, "queue")]

    for build_type, automated in build_types:
        for function, metric_type in metrics:
            for text, period in build_periods:
                metric = "Average %s %s time (%s)" % (build_type, metric_type,
                                                      text)
                results.append((metric,
                                function(period=period, automated=automated)))

    return results


def calculate_days(start, end):
    """
    Returns the number of days between two datetimes, rounded up.
    """
    delta = end - start
    seconds = 86400 * delta.days
    seconds += delta.seconds
    return round(seconds / 86400.0, 0)


def calculate_start_and_end(start, end):
    """
    Given up to two date strings, return a tuple of start/end date to query
    metrics from.

    The start date defaults to YYYY-01-01:00:00:00 of the current year.
    The end date defaults to YYYY-12-31:23:59:59 of the current year.
    """
    try:
        start_date = parse_pseudo_iso8601(start)
    except ValueError:
        raise ValueError("Invalid start date %s" % start)
    try:
        end_date = parse_pseudo_iso8601(end, end=True)
    except ValueError:
        raise ValueError("Invalid end date %s" % end)
    return (start_date, end_date)


def parse_pseudo_iso8601(date, end=False):
    """
    Parses a date from Murphy's pseudo-ISO 8601 format.
    """
    parsed_date = dateutil.parser.parse(date)
    element_count = len(re.split("[,-]", date))
    if element_count == 2:
        if end:
            start_of_next_month = datetime(parsed_date.year, parsed_date.month+1, 1)
            parsed_date = start_of_next_month - timedelta(days=1)
        else:
            parsed_date = datetime(parsed_date.year, parsed_date.month, 1)
    elif element_count == 1:
        if end:
            start_of_next_year = datetime(parsed_date.year + 1, 1, 1)
            parsed_date = start_of_next_year - timedelta(days=1)
        else:
            parsed_date = datetime(parsed_date.year, 1, 1)
    if end:
        parsed_date = datetime.combine(parsed_date.date(), time(23, 59, 59))
    return parsed_date
