# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import unittest
from datetime import datetime, timedelta

from django.test import TestCase

from offspring.web.queuemanager.metrics import (
    get_build_result_metrics, calculate_start_and_end,
    calculate_days, parse_pseudo_iso8601)

from offspring.web.queuemanager.models import BuildResult
from offspring.enums import ProjectBuildStates

from offspring.web.queuemanager.tests.factory import factory
from offspring.web.queuemanager.tests.helpers import (
    create_build_results_with_durations, create_build_results_with_queue_times)


class GetBuildResultsTests(TestCase):

    def setUp(self):
        self.project = factory.make_project()
        self.user = factory.make_user()

    def create_test_data(self, user=None, offset=0):
        """
        Creates tests data with the specified requestor user and datetime offset.
        """
        start_date = datetime.utcnow()
        last_week = start_date - timedelta(days=8)
        last_month = start_date - timedelta(days=35)
        # Two this week, average = 10 + 20/2 = 15
        create_build_results_with_durations(
            self.project, [10 + offset, 20],
            build_date=start_date - timedelta(days=1),
            requestor=user)

        # Two this week, average = 30 + 18/2 = 24.0
        create_build_results_with_queue_times(
            self.project, [30 + offset, 18],
            build_date=start_date - timedelta(days=1),
            requestor=user)

        # Two last week, 10 + 10 + 10 + 20/4 = 12.5
        create_build_results_with_durations(
            self.project, [10 + offset, 10], build_date=last_week,
            requestor=user)

        # Two last week, average = 30 + 18 + 14  11/4 = 73
        create_build_results_with_queue_times(
            self.project, [14 + offset, 11], build_date=last_week,
            requestor=user)

        # Two last month, 10 + 10 + 10 + 20 + 30 + 40/6 = 20
        create_build_results_with_durations(
            self.project, [30 + offset, 40], build_date=last_month,
            requestor=user)

        # Two last month, average = 30 + 18 + 14  11 + 30 + 20/6 = 82
        create_build_results_with_queue_times(
            self.project, [30 + offset, 20], build_date=last_month,
            requestor=user)

    def test_get_build_result_metrics(self):
        """
        get_build_result_metrics should return a list of tuples with the metric
        being measured, and the time in minutes.
        """
        self.create_test_data()
        self.create_test_data(user=self.user, offset=10)

        self.assertEqual(
            [("Average scheduled build time (7 days)", 15.0 * 60),
             ("Average scheduled build time (30 days)", 12.5 * 60),
             ("Average scheduled build time (90 days)", 20.0 * 60),
             ("Average scheduled queue time (7 days)", 24.0 * 60),
             ("Average scheduled queue time (30 days)", 18.25 * 60),
             ("Average scheduled queue time (90 days)", 20.5 * 60),
             ("Average requested build time (7 days)", 20.0 * 60),
             ("Average requested build time (30 days)", 17.5 * 60),
             ("Average requested build time (90 days)", 25.0 * 60),
             ("Average requested queue time (7 days)", 29.0 * 60),
             ("Average requested queue time (30 days)", 23.25 * 60),
             ("Average requested queue time (90 days)", 25.5 * 60)],
            get_build_result_metrics())

    def test_get_build_result_metrics_with_start_and_end_date(self):
        """
        get_build_result_metrics should return a list of tuples with the metric
        being measured, and the time in minutes.
        """
        self.create_test_data()
        self.create_test_data(user=self.user, offset=10)

        end_date = datetime.utcnow()
        start_date = end_date - timedelta(days=10)

        self.assertEqual(
            [("Average scheduled build time (10 days)", 12.5 * 60),
             ("Average scheduled queue time (10 days)", 18.25 * 60),
             ("Average requested build time (10 days)", 17.5 * 60),
             ("Average requested queue time (10 days)", 23.25 * 60),],
            get_build_result_metrics(start_date, end_date))


class ProjectGroupBuildMetricsTests(TestCase):

    def setUp(self):
        self.project_group = factory.make_project_group()
        self.project = factory.make_project(project_group=self.project_group)

    def test_get_average_build_time_with_project_group(self):
        """
        get_average_build_time should take a project group, and calculate the
        average build time of all projects within that group.
        """
        build_result = factory.make_build_result(
            project=self.project, result=ProjectBuildStates.SUCCESS)
        finished_at = datetime.now()
        build_result.finished_at = finished_at
        build_result.started_at = finished_at - timedelta(minutes=10)
        build_result.save()
        self.assertEqual(
            10 * 60,
            BuildResult.metrics.get_average_build_time(
                project_group=self.project_group))

    def test_get_average_build_time_with_multiple_projects_in_group(self):
        """
        All Projects in a ProjectGroup should be included in the average
        buildtime.
        """
        create_build_results_with_durations(self.project, [10, 20])
        project2 = factory.make_project(project_group=self.project_group)
        create_build_results_with_durations(project2, [20, 30])
        # 10 + 20 + 20 + 30 / 4 = 20 minutes
        self.assertEqual(
            20 * 60,
            BuildResult.metrics.get_average_build_time(
                project_group=self.project_group))

    def test_get_average_build_time_with_excludes_projects_not_in_group(self):
        """
        Only Projects in a ProjectGroup should be included in the average
        buildtime.
        """
        create_build_results_with_durations(self.project, [10, 20])
        project2 = factory.make_project()
        create_build_results_with_durations(project2, [20, 30])
        # 10 + 20  / 2 = 15 minutes
        self.assertEqual(
            15 * 60,
            BuildResult.metrics.get_average_build_time(
                project_group=self.project_group))

    def test_get_average_queue_time_with_project_group(self):
        """
        get_average_queue_time should take a project group, and calculate the
        average queue time of all projects within that group.
        """
        create_build_results_with_queue_times(self.project, [10, 20])
        self.assertEqual(
            15 * 60,
            BuildResult.metrics.get_average_queue_time(
                project_group=self.project_group))


class MetricsDateCalculatorTestCase(unittest.TestCase):

    def test_start_end_dates(self):
        """
        * with a start value of YYYY-MM-DD, the actual value will be YYYY-MM-DD
          00:00:00
        * with a start value of YYYY, the actual value will be YYYY-01-01 00:00:00
        * with an end value of YYYY-MM-DD, the actual value will be YYYY-MM-DD
          23:59:59
        * with a end value of YYYY, the actual value will be YYYY-12-31 23:59:59
        """
        self.assertEqual(
            (datetime(2013, 06, 05, 0, 0, 0), datetime(2013, 06, 20, 23, 59, 59)),
            calculate_start_and_end("2013-06-05", "2013-06-20"))
        self.assertEqual(
            (datetime(2013, 01, 01, 0, 0, 0), datetime(2013, 12, 31, 23, 59, 59)),
            calculate_start_and_end("2013", "2013"))
        self.assertEqual(
            (datetime(2013, 05, 01, 0, 0, 0), datetime(2013, 10, 31, 23, 59, 59)),
            calculate_start_and_end("2013-05", "2013-10"))
        self.assertEqual(
            (datetime(2013, 06, 05, 0, 0, 0), datetime(2013, 06, 05, 23, 59, 59)),
            calculate_start_and_end("2013-06-05", "2013-06-05"))

    def test_parse_errors(self):
        """
        calculate_start_and_end should return a sensible message if an error
        occurs during parsing.
        """
        with self.assertRaises(ValueError) as cm:
            calculate_start_and_end("TESTING", "2013-06-05")
        self.assertEqual(cm.exception.message, "Invalid start date TESTING")

        with self.assertRaises(ValueError) as cm:
            calculate_start_and_end("2013-06-05", "TESTING")
        self.assertEqual(cm.exception.message, "Invalid end date TESTING")

    def test_calculate_days(self):
        """
        calculate_days should calculate the number of days between two
        timestamps, rounding up, rather than rounding down as timedeltas do.
        """
        start = datetime(2013, 01, 01, 00, 00, 00)
        end = datetime(2013, 12, 31, 23, 59, 59)
        self.assertEqual(365, calculate_days(start, end))


class ParsePseudoISO8601TestCase(unittest.TestCase):

    def test_parse_pseudo_iso8601_start(self):
      """
      We should calculate the date in various permutations.
      """
      self.assertEqual(
          datetime(2013, 06, 05, 0, 0, 0), parse_pseudo_iso8601("2013-06-05"))
      self.assertEqual(
          datetime(2013, 06, 01, 0, 0, 0), parse_pseudo_iso8601("2013-06"))
      self.assertEqual(
          datetime(2013, 01, 01, 0, 0, 0), parse_pseudo_iso8601("2013"))

    def test_parse_pseudo_iso8601_end(self):
      """
      We should calculate the date in various permutations, if end is true, then
      the date calculated should be the end of the period specified.
      """
      self.assertEqual(
          datetime(2013, 06, 05, 23, 59, 59),
          parse_pseudo_iso8601("2013-06-05", True))
      self.assertEqual(
          datetime(2013, 06, 30, 23, 59, 59),
          parse_pseudo_iso8601("2013-06", True))
      self.assertEqual(
          datetime(2013, 12, 31, 23, 59, 59),
          parse_pseudo_iso8601("2013", True))
